﻿using CsvHelper;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Dynamic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApp1
{
    public partial class fMain : Form
    {
        public fMain()
        {
            InitializeComponent();
            Pub.init();
            this.Text = string.Format("{0} ver{1}", Application.ProductName, Application.ProductVersion);
        }

        private void fMain_Load(object sender, EventArgs e)
        {
            this.Show();
            Application.DoEvents();

            var dbfilename =  System.IO.Path.Combine( Util.CurrentPath ,"Database","MoneyBook.accdb");
            Properties.Settings.Default["CS"] = "Provider=Microsoft.ACE.OLEDB.12.0;Data Source=" + dbfilename;

            tbSD.Text = DateTime.Now.ToString("yyyy-MM-01");
            tbED.Text = DateTime.Now.AddMonths(1).AddDays(-1).ToShortDateString();

            if (System.IO.File.Exists(dbfilename)==false)
            {
                Util.MsgE("데이터베이스파일이 없습니다.\n" + dbfilename);
                this.Close();
            }
            else
            {
                //로그인 작업 (실패시 종료)
                if (func_Login() == false)
                    this.Close();

                //이번달 데이터 불러오기
                func_loaddata();
            }
        }
        Boolean func_Login()
        {
            var f = new fLogin();
            if (f.ShowDialog() == DialogResult.OK) return true;
            else return false;
        }
        void func_summary()
        {
            decimal prejan = this.dataSet1.Data.Where(t => t.RowState != DataRowState.Deleted && t.Grp == "이월").Sum(t => t.Dr);
            var sumDr = this.dataSet1.Data.Where(t => t.RowState != DataRowState.Deleted && t.Grp != "이월").Sum(t => t.Dr);
            var sumCr = this.dataSet1.Data.Where(t => t.RowState != DataRowState.Deleted && t.Grp != "이월").Sum(t => t.Cr);
            var jan = prejan + sumDr - sumCr;
            this.lbPreJan.Text = prejan.ToString("N0");
            this.lbSumDr.Text = sumDr.ToString("N0");
            this.lbSumCr.Text = sumCr.ToString("N0");
            this.lbSumJan.Text = jan.ToString("N0");

            ChartRefresh();
        }
        void func_loaddata()
        {
            //기존 목록 삭제(=데이터)
            //this.dataSet1.Data.Clear();

            //lbMon.Text = mon;
            //var dbfile = Util.CurrentPath + "Database\\" + mon + ".xml";
            //Console.WriteLine("read file : " + dbfile);

            //불러오기
            try
            {
                //2020-09
                var 시작일 = DateTime.Parse(tbSD.Text + " 00:00:00");
                var 종료일 = DateTime.Parse(tbED.Text + " 23:59:59");// 시작일.AddMonths(1).AddDays(-1).ToShortDateString() + " 23:59:59");
                taData.Fill(this.dataSet1.Data, 시작일, 종료일);
            }
            catch (Exception ex)
            {
                Util.MsgE(ex.Message);
            }
            //if (System.IO.File.Exists(dbfile))
            //{
            //    this.dataSet1.Data.ReadXml(dbfile);
            //    this.dataSet1.Data.AcceptChanges();
            //}
            func_summary();
        }
        void func_savedata()
        {
            var mon = lbMon.Text;
            try
            {
                taData.Update(this.dataSet1.Data);
            }
            catch (Exception ex)
            {
                Util.MsgE(ex.Message);
            }
            //var dbfile = Util.CurrentPath + "Database\\" + mon + ".xml";
            //Console.WriteLine("save file : " + dbfile);
            //this.dataSet1.Data.WriteXml(dbfile);
        }

        private void 마감ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //현재선택된 데이터의 마감작업

            var lockCount = this.dataSet1.Data.Where(t => t.RowState != DataRowState.Deleted && t.Lock == true).Count();
            if (lockCount > 0)
            {
                var dlg = Util.MsgQ("월 마감을 해제 하시겠습니까?");
                if (dlg == DialogResult.Yes)
                {
                    //마감해제
                    func_mon_reject();
                }
            }
            else
            {
                var dlg = Util.MsgQ("월 마감을 하시겠습니까?");
                if (dlg == DialogResult.Yes)
                {
                    func_mon_commit();
                }
            }
        }

        void func_mon_commit()
        {
            //이번달의 잔액을 다음달의 1일에 "이월" 항목으로 입금액에 추가한다
            var currentMon = DateTime.Parse(lbMon.Text + "-01");

            var nextMon = currentMon.AddMonths(1);
            var nextFile = Util.CurrentPath + "Database\\" + nextMon.ToString("yyyy-MM") + ".xml";

            var dsNext = new DataSet1();
            if (System.IO.File.Exists(nextFile) == false)
            {
                var newdr = dsNext.Data.NewDataRow();
                newdr.PDate = nextMon;
                newdr.Grp = "이월";
                newdr.Dr = decimal.Parse(lbSumJan.Text.Replace(",", ""));
                newdr.EndEdit();
                dsNext.Data.AddDataRow(newdr);
            }
            else
            {
                //파일이 존재할때 - 이월데이터를 변경 해준다(없으면 생성)
                dsNext.Data.ReadXml(nextFile);
                var sumDr = dsNext.Data.Where(t => t.Grp == "이월").FirstOrDefault();
                if (sumDr == null)
                {
                    var newdr = dsNext.Data.NewDataRow();
                    newdr.PDate = nextMon;
                    newdr.Grp = "이월";
                    newdr.Dr = decimal.Parse(lbSumJan.Text.Replace(",", ""));
                    newdr.EndEdit();
                    dsNext.Data.AddDataRow(newdr);
                }
                else
                {
                    sumDr.PDate = nextMon;
                    sumDr.Grp = "이월";
                    sumDr.Dr = decimal.Parse(lbSumJan.Text.Replace(",", ""));
                    sumDr.EndEdit();
                }
            }
            dsNext.Data.AcceptChanges();
            dsNext.Data.WriteXml(nextFile);


            //모든 데이터의 lock을 설정한다
            for (int i = 0; i < this.dataSet1.Data.Rows.Count; i++)
            {
                var dr = this.dataSet1.Data.Rows[i] as DataSet1.DataRow;
                dr.Lock = true;
                dr.EndEdit();
            }
            this.dataSet1.Data.AcceptChanges();
            func_savedata();

            //다음달 데이터 자동 로딩
            tbSD.Text = nextMon.ToString("yyyy-MM-01");
            tbED.Text = nextMon.AddMonths(1).AddDays(-1).ToString("yyyy-MM-dd");
            func_loaddata();
            Pub.log.Add("마감 " + lbMon.Text + ",잔액=" + lbSumJan.Text);
        }
        void func_mon_reject()
        {
            //다음달 1일의 이월 항목도 삭제를 하고, 
            var currentMon = DateTime.Parse(lbMon.Text + "-01");
            var nextMon = currentMon.AddMonths(1);
            var nextFile = Util.CurrentPath + "Database\\" + nextMon.ToString("yyyy-MM") + ".xml";

            var dsNext = new DataSet1();
            if (System.IO.File.Exists(nextFile))
            {
                dsNext.Data.ReadXml(nextFile);
                var sumDr = dsNext.Data.Where(t => t.Grp == "이월").FirstOrDefault();
                if (sumDr != null) sumDr.Delete();
            }
            dsNext.Data.AcceptChanges();
            dsNext.Data.WriteXml(nextFile);

            //모든데이터의 lock을 해제한다
            foreach (DataSet1.DataRow dr in this.dataSet1.Data.Rows)
            {
                if (dr.RowState != DataRowState.Deleted) dr.Lock = false;
            }
            this.dataSet1.Data.AcceptChanges();
            func_savedata();
            Pub.log.Add("마감해제 " + lbMon.Text);
        }

        private void 로그인ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (func_Login() == false) this.Close();
        }

        private void dataBindingNavigatorSaveItem_Click(object sender, EventArgs e)
        {
            func_savedata();
        }

        private void bindingNavigatorAddNewItem_Click(object sender, EventArgs e)
        {
            func_add();

        }

        void func_delete()
        {
            var drv = bs.Current as DataRowView;
            if (drv == null)
            {
                Util.MsgE("삭제할 자료를 선택하세요");
                return;
            }

            var dr = drv.Row as DataSet1.DataRow;
            if (dr.Lock)
            {
                Util.MsgE("마감된 자료는 삭제할 수 없습니다");
                return;
            }
            if (dr.Grp == "이월")
            {
                Util.MsgE("이월 데이터는 삭제할 수 없습니다");
                return;
            }


            //현재 선택된 자료를 삭제합니다.
            var dlg = Util.MsgQ("현재 자료를 삭제하시겠습니까?");
            if (dlg != DialogResult.Yes) return;


            //현재데이터 삭제
            bs.RemoveCurrent();
            func_summary();
        }
        void func_edit()
        {
            //새로운 데이터 생성
            var drv = this.bs.Current as DataRowView;
            if (drv == null)
            {
                Util.MsgE("선택된 자료가 없습니다");
                return;
            }

            var curdr = drv.Row as DataSet1.DataRow;
            if (curdr.Grp == "이월")
            {
                Util.MsgE("이월 데이터는 편집할 수 없습니다");
                return;
            }

            var f = new fItem(curdr);
            if (f.ShowDialog() == DialogResult.OK)
            {
                //추가(입금 ? 출금)
                curdr.EndEdit();
            }
            else curdr.RejectChanges();
            func_summary();
        }
        void func_add()
        {
            //새로운 데이터 생성
            var newdr = this.dataSet1.Data.NewDataRow();
            newdr.PDate = DateTime.Now;

            var f = new fItem(newdr);
            if (f.ShowDialog() == DialogResult.OK)
            {
                //추가(입금 ? 출금)
                newdr.EndEdit();
                if (newdr.Grp == "이월")
                {
                    Util.MsgE("이월 데이터는 생성할 수 없습니다");
                    newdr.Delete();
                }
                else this.dataSet1.Data.AddDataRow(newdr);
            }
            else newdr.Delete();
            func_summary();
        }
        private void bindingNavigatorDeleteItem_Click(object sender, EventArgs e)
        {
            //삭제
            func_delete();
        }

        private void dataDataGridView_DoubleClick(object sender, EventArgs e)
        {
            func_edit();
        }

        private void cSV저장ToolStripMenuItem_Click(object sender, EventArgs e)
        {


            var sd = new SaveFileDialog();
            sd.Filter = "CSV File(.csv)|*.csv";
            sd.FileName = "export_" + DateTime.Now.ToString("yyyyMMddHHmmss") + ".csv";
            if (sd.ShowDialog() == DialogResult.OK)
            {

                var records = new List<dynamic>();

                var sortedrows = this.dataSet1.Data.Select("", bs.Sort);
                for (int i = 0; i < sortedrows.Length; i++)
                {
                    var dr = sortedrows[i] as DataSet1.DataRow;

                    dynamic record = new ExpandoObject();
                    record.날짜 = dr.PDate.ToShortDateString(); //yyyy-MM-dd
                    record.구분 = dr.Grp;
                    record.입금액 = dr.Dr.ToString("N0");
                    record.출금액 = dr.Cr.ToString("N0");
                    record.비고 = dr.Remark;
                    records.Add(record);

                }

                using (var writer = new StringWriter())
                using (var csv = new CsvWriter(writer, CultureInfo.InvariantCulture))
                {
                    csv.WriteRecords(records);
                    var contents = writer.ToString();

                    //buffer를 저장한다
                    try
                    {
                        System.IO.File.WriteAllText(sd.FileName, contents, System.Text.Encoding.UTF8);
                        Pub.log.Add("파일내보내기 완료" + sd.FileName);
                    }
                    catch (Exception ex)
                    {
                        Util.MsgE(ex.Message);
                        Pub.log.AddE(ex.Message);
                    }

                }





            }
        }


        private void dataDataGridView_Sorted(object sender, EventArgs e)
        {
            var sortcol = this.dataDataGridView.SortedColumn;
            var fieldName = sortcol.DataPropertyName;
            var sortOrder = dataDataGridView.SortOrder;
            if (sortOrder == SortOrder.Ascending) this.bs.Sort = fieldName;
            else if (sortOrder == SortOrder.Descending) this.bs.Sort = fieldName + " desc";
            Console.WriteLine("bs sort=" + this.bs.Sort);
        }

        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            ChartRefresh();
        }
        void ChartRefresh()
        {
            //데이터는 시리즈의 point 에 입력합니다.
            //범례는 legend 를 수정

            //1.번차트 (그룹별, 지출내역)
            var grplist1 = this.dataSet1.Data.Where(t=>t.RowState != DataRowState.Deleted).GroupBy(t => t.Grp);
            chart1.Series[0].Points.Clear();
            foreach (var item in grplist1)
            {
                var GrpName = item.FirstOrDefault().Grp;
                var GrpSum = item.Sum(t => t.Cr);
                chart1.Series[0].Points.AddXY(GrpName, GrpSum);
            }

            //2.번차트 (일자별, 지출내역)
            var grplist2 = this.dataSet1.Data.Where(t=> t.RowState != DataRowState.Deleted).GroupBy(t => t.PDate.ToShortDateString());
            chart2.Series[0].Points.Clear();
            foreach (var item in grplist2)
            {
                var GrpName = item.FirstOrDefault().PDate.ToString("MM-dd");
                var GrpSum = item.Sum(t => t.Cr);
                chart2.Series[0].Points.AddXY(GrpName, GrpSum);
            }
        }

        private void 조회ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            func_loaddata();
        }

        private void 사용자관리ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var f = new fUsers();
            f.ShowDialog();
        }
    }
}
